const cloneDeep = require('clone-deep');
const getUUID = require('uuid-by-string');

class Thread {
  constructor(name, options) {
    var self = this;
    self.options = options;

    self.name = name + '';

    if (!options || (options.fault !== false && options.fault !== true))
      self.fault = true;

    self.callstack = []
  }

  frame(f) {
    var self = this;
    var strip = [ "__interceptor_" ];
    var reset = [ "__interceptor_", "__asan", "__sanitizer" ];
    var ignore = [ "__libc_start_main", "start_thread", "thread_start", "__clone",
      "__asan::AsanThread::ThreadStart" ];

    if (!f["function"] || f["function"] === "") {
      
      // Keep the frame as unsymbolicated
      if (f["address"] && f["address"] !== "") {
        self.callstack.push(f);
        return f;
      } else {
        return;
      } 
    } 

    for (var i = 0; i < strip.length; i++) {
      f["function"] = f["function"].replace(strip[i], "");
    }

    for (var i = 0; i < ignore.length; i++) {
      if (f["function"].indexOf(ignore[i]) === 0) {
        return;
      }
    }

    if (self.options && self.options.reset === false)
      return;

    for (var i = 0; i < reset.length; i++) {
      if (f["function"].indexOf("AsanThread") >= 0)
        break;

      if (f["function"].indexOf(reset[i]) === 0) {
        self.callstack = [];
      }
    }
    self.callstack.push(f);
    return f;
  }
  
}

class report {
  checksum(d) {
    this.checksum = getUUID(d + '');
    this.report.attributes.uuid = this.checksum;
    return;
  }

  serializeReport(r) {
    var self = this;
    var rp = {};
    var counter = 0;

    if (r.report.attributes['timestamp'] >= 0) {
      rp.timestamp = r.report.attributes['timestamp'];
    } else {
      rp.timestamp = parseInt(Date.now() / 1000);
    }

    rp.agent = "mela";
    rp.lang = "C";
    rp.langVersion = '1.0.0';
    rp.threads = {};
    rp.sourceCode = {};
    rp.classifiers = r.report.classifiers;
    rp.attributes = r.report.attributes;

    /* Lift UUID from an attribute and shape it for the bad BU. */
    if (!self.checksum) {
      console.error('Warning: report.checksum has not been called.');
      rp.uuid = '123e4567-e89b-12d3-a456-426655440000';
    } else {
      rp.uuid = self.checksum;
      if (!rp.attributes.guid)
        rp.attributes.guid = rp.uuid;
    }

    rp.annotations = r.report.annotations;

    for (var k in r.report.threads) {
      var th = r.report.threads[k];
      var ch = rp.threads[k] = {};

      ch.name = th.name;
      ch.fault = th.fault;
      if (ch.fault === true && !rp.mainThread)
        rp.mainThread = th.name;

      ch.stack = [];

      for (var i = 0; i < th.callstack.length; i++) {
        var thf = th.callstack[i];

        var fr = {
          'guessed_frame' : false,
          'library' : thf['object']
        };

        if (thf['function'] && thf['function'] !== '')
          fr.funcName = thf['function'];
      
        if (thf['address'] && thf['address'] !== '')
          fr.address = thf['address'];

        if (fr.library && rp.attributes && rp.attributes['_mod_tolower']) {
          fr.library = fr.library.toLowerCase();
        }

        var line;
        try {
          line = parseInt(thf['line']);
        } catch (error) {}

        if (line)
          fr.line = line;

        if (thf.registers)
          fr.registers = thf.registers;

        if (thf.file || thf.path || thf.text) {
          var sc = {
            "type" : "text",
            "title" : "Log File",
            "highlightLine" : false
          };

          if (thf.file) {
            sc.path = thf.file;
          } else if (thf.path) {
            sc.path = thf.path;
          }

          if (sc.path && rp.attributes && rp.attributes['_mod_tolower'])
            sc.path = sc.path.toLowerCase();

          if (thf.text)
            sc.text = thf.text;

          fr.sourceCode = counter++ + '';
          rp.sourceCode[fr.sourceCode] = sc;
        }

        ch.stack.push(fr);
      }
    }

    return rp;
  }

  serialize() {
    var self = this;
    var reports = [];

    for (var i = 0; i < self.reports.length; i++) {
      var r = self.serializeReport(self.reports[i]);

      if (!r.mainThread || r.mainThread === "")
        continue;

      reports.push(r);
    }

    return reports;
  }

  constructor(report) {
    var self = this;

    if (report) {
      self.report = cloneDeep(report.report);
      self.checksum = report.checksum;
      report.reports.push(self);
    } else {
      self.report = {
        threads : {},
        annotations : {},
        attributes : {},
        classifiers : []
      };
    }

    /* This is an array of child reports. */
    self.reports = [];
  }

  thread(name) {
    var self = this;

    var thread = new Thread(name);
    self.report.threads[name] = thread;

    return thread;
  }

  set(type, a, b) {
    var self = this;

    if (type === 'annotation') {
      for (var j in a) {
        self.report.annotations[j] = a[j];
      }
    } else if (type === 'attribute') {
      self.report.attributes[a] = b;
    }

    return self;
  }

  add(type, a, b) {
    var self = this;

    if (type === 'classifiers') {
      self.report.classifiers.push(a);      
    }

    return self;
  }

}

module.exports = report;

//-- vim:ts=2:et:sw=2:st=2
