"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Events = void 0;
class Events {
    constructor() {
        this._callbacks = {};
    }
    on(event, callback) {
        this.addCallback(event, { callback });
        return this;
    }
    once(event, callback) {
        this.addCallback(event, { callback, once: true });
        return this;
    }
    off(event, callback) {
        this.removeCallback(event, callback);
        return this;
    }
    emit(event, ...args) {
        const callbacks = this._callbacks[event];
        if (!callbacks || !callbacks.length) {
            return false;
        }
        for (const { callback, once } of [...callbacks]) {
            try {
                callback(...args);
            }
            catch (_a) {
                // Do nothing
            }
            if (once) {
                this.removeCallback(event, callback);
            }
        }
        return true;
    }
    addCallback(event, callback) {
        const list = this._callbacks[event];
        if (list) {
            list.push(callback);
        }
        else {
            this._callbacks[event] = [callback];
        }
    }
    removeCallback(event, callback) {
        const list = this._callbacks[event];
        if (!list) {
            return;
        }
        const index = list.findIndex((el) => el.callback === callback);
        if (index === -1) {
            return;
        }
        list.splice(index, 1);
        if (!list.length) {
            delete this._callbacks[event];
        }
    }
}
exports.Events = Events;
//# sourceMappingURL=Events.js.map