"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.jsonEscaper = void 0;
function jsonEscaper() {
    const ancestors = [];
    const keys = [];
    // in TypeScript add "this: any" param to avoid compliation errors - as follows
    //    return function (this: any, field: any, value: any) {
    return function (key, value) {
        if (!key) {
            return value;
        }
        if (value === null) {
            return value;
        }
        const valueType = typeof value;
        if (valueType === 'bigint') {
            return value.toString();
        }
        if (valueType !== 'object') {
            return value;
        }
        // `this` is the object that value is contained in,
        // i.e., its direct parent.
        while (ancestors.length > 0 && ancestors[ancestors.length - 1] !== this) {
            ancestors.pop();
            keys.pop();
        }
        if (ancestors.includes(value)) {
            return `[Circular].${keys.join('.')}.${key}`;
        }
        keys.push(key);
        ancestors.push(value);
        return value;
    };
}
exports.jsonEscaper = jsonEscaper;
//# sourceMappingURL=jsonEscaper.js.map