"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BacktraceReport = void 0;
const jsonEscaper_1 = require("../../common/jsonEscaper");
const TimeHelper_1 = require("../../common/TimeHelper");
class BacktraceReport {
    addStackTrace(name, stack, message = '') {
        if (typeof stack === 'string') {
            this.stackTrace[name] = {
                stack,
                message,
            };
        }
        else {
            this.stackTrace[name] = stack;
        }
        return this;
    }
    constructor(data, attributes = {}, attachments = [], options = {}) {
        var _a, _b, _c;
        this.data = data;
        this.attributes = attributes;
        this.attachments = attachments;
        /**
         * Report classifiers
         */
        this.classifiers = [];
        /**
         * Report annotations
         */
        this.annotations = {};
        /**
         * Report stack trace
         */
        this.stackTrace = {};
        /**
         * Report inner errors
         */
        this.innerReport = [];
        /**
         * Report timestamp in ms
         */
        this.timestamp = TimeHelper_1.TimeHelper.now();
        /**
         * Sets how many top frames should be skipped.
         */
        this.skipFrames = 0;
        this.skipFrames = (_a = options === null || options === void 0 ? void 0 : options.skipFrames) !== null && _a !== void 0 ? _a : 0;
        let errorType = 'Exception';
        if (data instanceof Error) {
            this.message = this.generateErrorMessage(data.message);
            this.annotations['error'] = Object.assign(Object.assign({}, data), { message: this.message, name: data.name, stack: data.stack });
            this.classifiers = [data.name];
            this.stackTrace['main'] = {
                stack: (_b = data.stack) !== null && _b !== void 0 ? _b : '',
                message: this.message,
            };
            // Supported in ES2022
            if (data.cause) {
                this.innerReport.push(data.cause);
            }
        }
        else {
            this.message = this.generateErrorMessage(data);
            this.stackTrace['main'] = {
                stack: (_c = new Error().stack) !== null && _c !== void 0 ? _c : '',
                message: this.message,
            };
            this.classifiers = ['Message'];
            errorType = 'Message';
            this.skipFrames += 1;
        }
        if (!this.attributes['error.type']) {
            this.attributes['error.type'] = errorType;
        }
        this.attributes['error.message'] = this.message;
        if (options === null || options === void 0 ? void 0 : options.timestamp) {
            this.timestamp = options.timestamp;
        }
        if (options === null || options === void 0 ? void 0 : options.classifiers) {
            this.classifiers.unshift(...options.classifiers);
        }
    }
    generateErrorMessage(data) {
        var _a;
        return typeof data === 'object' ? JSON.stringify(data, (0, jsonEscaper_1.jsonEscaper)()) : (_a = data === null || data === void 0 ? void 0 : data.toString()) !== null && _a !== void 0 ? _a : '';
    }
}
exports.BacktraceReport = BacktraceReport;
//# sourceMappingURL=BacktraceReport.js.map