"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AttributeManager = void 0;
const Events_1 = require("../../common/Events");
const ReportDataBuilder_1 = require("./ReportDataBuilder");
class AttributeManager {
    constructor(providers) {
        this._attributeProviders = [];
        this.attributeEvents = new Events_1.Events();
        for (const provider of providers) {
            this.addProvider(provider);
        }
    }
    /**
     * Adds attributes to manager cache
     * @param attributes attributes object
     */
    add(attributes) {
        if (typeof attributes === 'function') {
            this.addProvider({ type: 'dynamic', get: attributes });
        }
        else {
            this.addProvider({ type: 'scoped', get: () => attributes });
        }
    }
    /**
     * Adds attribute provider to the manager
     * @param attributeProvider
     * @returns
     */
    addProvider(attributeProvider) {
        if (attributeProvider.type === 'dynamic') {
            this._attributeProviders.push(attributeProvider);
            return;
        }
        else {
            const attributes = attributeProvider.get();
            this._attributeProviders.push({
                type: 'scoped',
                get: () => attributes,
            });
            this.attributeEvents.emit('scoped-attributes-updated', this.get('scoped'));
        }
    }
    /**
     * Gets client attributes
     * @returns Report attribute - client attributes and annotations
     */
    get(attributeType) {
        const result = {
            annotations: {},
            attributes: {},
        };
        for (const attributeProvider of this._attributeProviders) {
            if (attributeType && attributeProvider.type != attributeType) {
                continue;
            }
            const providerResult = ReportDataBuilder_1.ReportDataBuilder.build(attributeProvider.get());
            result.attributes = Object.assign(Object.assign({}, result.attributes), providerResult.attributes);
            result.annotations = Object.assign(Object.assign({}, result.annotations), providerResult.annotations);
        }
        return result;
    }
}
exports.AttributeManager = AttributeManager;
//# sourceMappingURL=AttributeManager.js.map