"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileAttributeManager = void 0;
const jsonEscaper_1 = require("../../common/jsonEscaper");
const ATTRIBUTE_FILE_NAME = 'bt-attributes';
class FileAttributeManager {
    constructor(_fileSystem, _fileName) {
        this._fileSystem = _fileSystem;
        this._fileName = _fileName;
    }
    static create(fileSystem) {
        return new FileAttributeManager(fileSystem);
    }
    static createFromSession(sessionFiles, fileSystem) {
        const fileName = sessionFiles.getFileName(ATTRIBUTE_FILE_NAME);
        return new FileAttributeManager(fileSystem, fileName);
    }
    initialize() {
        this.saveAttributes();
    }
    bind({ attributeManager, sessionFiles }) {
        if (this._fileName) {
            throw new Error('This instance is already bound.');
        }
        if (!sessionFiles) {
            return;
        }
        this._fileName = sessionFiles.getFileName(ATTRIBUTE_FILE_NAME);
        this._attributeManager = attributeManager;
        attributeManager.attributeEvents.on('scoped-attributes-updated', () => this.saveAttributes());
    }
    dispose() {
        this._fileName = undefined;
    }
    get() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._fileName) {
                return {};
            }
            try {
                const content = yield this._fileSystem.readFile(this._fileName);
                return JSON.parse(content);
            }
            catch (_a) {
                return {};
            }
        });
    }
    saveAttributes() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._fileName || !this._attributeManager) {
                return;
            }
            const reportData = this._attributeManager.get('scoped');
            yield this._fileSystem.writeFile(this._fileName, JSON.stringify(reportData.attributes, (0, jsonEscaper_1.jsonEscaper)()));
        });
    }
}
exports.FileAttributeManager = FileAttributeManager;
//# sourceMappingURL=FileAttributeManager.js.map