"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BacktraceDatabaseContext = void 0;
class BacktraceDatabaseContext {
    constructor(bucketCount = 3) {
        this.bucketCount = bucketCount;
        this.recordBucket = this.setupRecordBucket(this.bucketCount);
    }
    find(predicate) {
        for (let index = 0; index < this.bucketCount; index++) {
            for (const record of this.recordBucket[index]) {
                if (predicate(record)) {
                    return record;
                }
            }
        }
        return undefined;
    }
    add(record) {
        this.recordBucket[0].push(record);
    }
    get() {
        const result = [];
        for (const bucket of this.recordBucket) {
            result.push(...bucket.map((n) => {
                return Object.assign({}, n);
            }));
        }
        return result;
    }
    getBucket(index) {
        return this.recordBucket[index];
    }
    count() {
        return Object.values(this.recordBucket)
            .map((n) => n.reduce((total, record) => total + record.count, 0))
            .reduce((total, current) => total + current, 0);
    }
    remove(databaseRecord) {
        for (let bucketIndex = 0; bucketIndex < this.bucketCount; bucketIndex++) {
            for (let recordIndex = 0; recordIndex < this.recordBucket[bucketIndex].length; recordIndex++) {
                const record = this.recordBucket[bucketIndex][recordIndex];
                if (databaseRecord.id === record.id) {
                    this.recordBucket[bucketIndex].splice(recordIndex, 1);
                    return;
                }
            }
        }
    }
    increaseBucket(bucketStart) {
        for (let bucketIndex = this.bucketCount - 1; bucketIndex >= bucketStart; bucketIndex--) {
            if (bucketIndex === this.bucketCount - 1) {
                this.recordBucket[bucketIndex] = [];
                continue;
            }
            this.recordBucket[bucketIndex + 1] = this.recordBucket[bucketIndex];
            this.recordBucket[bucketIndex] = [];
        }
    }
    load(records) {
        this.recordBucket[0].push(...records);
    }
    dropOverflow(overflow) {
        const result = [];
        for (let bucketIndex = this.bucketCount - 1; bucketIndex >= 0; bucketIndex--) {
            const bucket = this.recordBucket[bucketIndex];
            const removedRecords = bucket.splice(0, overflow);
            result.push(...removedRecords);
            if (result.length === overflow) {
                break;
            }
        }
        return result;
    }
    setupRecordBucket(retries) {
        const result = [];
        for (let index = 0; index < retries; index++) {
            result[index] = [];
        }
        return result;
    }
}
exports.BacktraceDatabaseContext = BacktraceDatabaseContext;
//# sourceMappingURL=BacktraceDatabaseContext.js.map