"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetricsSubmissionQueue = void 0;
const AbortController_1 = require("../../common/AbortController");
const DelayHelper_1 = require("../../common/DelayHelper");
const TimeHelper_1 = require("../../common/TimeHelper");
const jsonEscaper_1 = require("../../common/jsonEscaper");
class MetricsSubmissionQueue {
    get total() {
        return this._events.length;
    }
    get submissionUrl() {
        return this._submissionUrl;
    }
    constructor(_submissionUrl, _eventName, _requestHandler, _metricMetadata, maximumEvents = 50) {
        this._submissionUrl = _submissionUrl;
        this._eventName = _eventName;
        this._requestHandler = _requestHandler;
        this._metricMetadata = _metricMetadata;
        this.maximumEvents = maximumEvents;
        this.DELAY_BETWEEN_REQUESTS = TimeHelper_1.TimeHelper.convertSecondsToMilliseconds(10);
        this._events = [];
        this._numberOfDroppedRequests = 0;
        this.MAXIMUM_NUMBER_OF_ATTEMPTS = 3;
        this._abortController = (0, AbortController_1.createAbortController)();
    }
    add(event) {
        this._events.push(event);
        if (this.reachedLimit()) {
            this.send();
        }
    }
    send(abortSignal) {
        return __awaiter(this, void 0, void 0, function* () {
            const eventsToProcess = this._events.splice(0);
            return yield this.submit(eventsToProcess, (0, AbortController_1.anySignal)(abortSignal, this._abortController.signal));
        });
    }
    dispose() {
        this._abortController.abort();
    }
    submit(events, abortSignal) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let attempts = 0; attempts < this.MAXIMUM_NUMBER_OF_ATTEMPTS; attempts++) {
                const response = yield this._requestHandler.post(this._submissionUrl, JSON.stringify(Object.assign(Object.assign({}, this._metricMetadata), { [this._eventName]: events, metadata: {
                        dropped_events: this._numberOfDroppedRequests,
                    } }), (0, jsonEscaper_1.jsonEscaper)()), abortSignal);
                if (response.status === 'Ok') {
                    this._numberOfDroppedRequests = 0;
                    return;
                }
                this._numberOfDroppedRequests++;
                yield DelayHelper_1.Delay.wait(Math.pow(2, attempts) * this.DELAY_BETWEEN_REQUESTS, abortSignal);
            }
            // if the code reached this line, it means, we couldn't send data to server
            // we need to try to return events to the queue and try to send it once again later.
            this.returnEventsIfPossible(events);
        });
    }
    returnEventsIfPossible(events) {
        if (this.maximumEvents < this._events.length + events.length) {
            return;
        }
        // push events to the beginning of the queue
        this._events.unshift(...events);
    }
    reachedLimit() {
        return this.maximumEvents === this._events.length && this.maximumEvents !== 0;
    }
}
exports.MetricsSubmissionQueue = MetricsSubmissionQueue;
//# sourceMappingURL=MetricsSubmissionQueue.js.map